diff --git a/src/http/ngx_http_upstream.c b/src/http/ngx_http_upstream.c
index dc3babd..10b5258 100644
--- a/src/http/ngx_http_upstream.c
+++ b/src/http/ngx_http_upstream.c
@@ -85,6 +85,9 @@ static ngx_int_t ngx_http_upstream_process_expires(ngx_http_request_t *r,
     ngx_table_elt_t *h, ngx_uint_t offset);
 static ngx_int_t ngx_http_upstream_process_accel_expires(ngx_http_request_t *r,
     ngx_table_elt_t *h, ngx_uint_t offset);
+static ngx_int_t ngx_http_upstream_process_connection(ngx_http_request_t *r,
+                                                      ngx_table_elt_t *h,
+                                                      ngx_uint_t offset);
 static ngx_int_t ngx_http_upstream_process_limit_rate(ngx_http_request_t *r,
     ngx_table_elt_t *h, ngx_uint_t offset);
 static ngx_int_t ngx_http_upstream_process_buffering(ngx_http_request_t *r,
@@ -220,7 +223,7 @@ ngx_http_upstream_header_t  ngx_http_upstream_headers_in[] = {
                  offsetof(ngx_http_headers_out_t, accept_ranges), 1 },

     { ngx_string("Connection"),
-                 ngx_http_upstream_ignore_header_line, 0,
+                 ngx_http_upstream_process_connection, 0,
                  ngx_http_upstream_ignore_header_line, 0, 0 },

     { ngx_string("Keep-Alive"),
@@ -3378,6 +3381,22 @@ ngx_http_upstream_process_accel_expires(ngx_http_request_t *r,


 static ngx_int_t
+ngx_http_upstream_process_connection(ngx_http_request_t *r,
+                                     ngx_table_elt_t *h,
+                                     ngx_uint_t offset) {
+
+    if (ngx_strlcasestrn(h->value.data, h->value.data + h->value.len,
+                         (u_char *) "close", sizeof("close") - 1)
+        != NULL)
+    {
+        r->upstream->headers_in.keepalive = 1;
+    }
+
+    return NGX_OK;
+}
+
+
+static ngx_int_t
 ngx_http_upstream_process_limit_rate(ngx_http_request_t *r, ngx_table_elt_t *h,
     ngx_uint_t offset)
 {
diff --git a/src/http/ngx_http_upstream.h b/src/http/ngx_http_upstream.h
index c915b26..8539c62 100644
--- a/src/http/ngx_http_upstream.h
+++ b/src/http/ngx_http_upstream.h
@@ -263,6 +263,7 @@ typedef struct {
     ngx_array_t                      cache_control;

     unsigned                         chunked:1;
+    unsigned                         keepalive:1;
 } ngx_http_upstream_headers_in_t;

